/*
* Kevin Yang
* CPE 471
* Program 2A
* Zoe Wood
* 1/23/2016
*/

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <time.h> 
#

#define GLEW_STATIC
#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include "GLSL.h"
#include "Program.h"
#include "MatrixStack.h"
#include "loadShader.h"

using namespace std;
using namespace Eigen;

GLFWwindow *window; // Main application window
string RESOURCE_DIR = "../resources/"; // Where the resources are loaded from
shared_ptr<Program> prog; //our shader program
const int worldWidth = 600;
const int worldHeight = 600; 
const int g_numP = 40;
float pointTime = 0.0;
float radius = 0.15;

static const GLfloat g_vertex_buffer_data[] = {
-1.0f, -1.0f, 0.0f,
1.0f, -1.0f, 0.0f,
0.0f, 1.0f, 0.0f
};

int main(int argc, char **argv)
{
	// Initiate glfw and check version
	if (!glfwInit())
	{
		fprintf(stderr, "Failed to initialize GLFW\n");
		return -1;
	}
	glfwWindowHint(GLFW_SAMPLES, 4);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 1);

	//Create OpenGL context window
	window = glfwCreateWindow(worldWidth, worldHeight, "Kevin Yang", NULL, NULL);
	if (!window) {
		fprintf(stderr, "Cannot open GLFW window.\n");
		glfwTerminate();
		return -1;
	}
	else {
		glfwMakeContextCurrent(window);
	}

	// Initiate glew
	if (glewInit() != GLEW_OK) {
		fprintf(stderr, "Cannot initialize GLEW\n");
		return -1;
	}
	
	// depth test or z-buffer
	glEnable(GL_DEPTH_TEST);

	// Vertex and Fragment shades
	GLuint progId = LoadShaders("SimpleVertexShader.glsl", "SimpleFragmentShader.glsl");
	GLuint ptId = glGetAttribLocation(progId, "points");
	GLuint backgroundCenterId = glGetUniformLocation(progId, "uniBackgroundCenter");
	GLuint radId = glGetUniformLocation(progId, "uniRadius");
	GLuint timeId = glGetUniformLocation(progId, "uniTime");
	GLuint windowId = glGetUniformLocation(progId, "uniWindowSize");

	int i; // Counter for number of points in buffer
	// Random array of points
	GLfloat pointsBuffer[g_numP * 20];
	srand(time(NULL));

	for (i = 0; i < g_numP * 3;) {
		pointsBuffer[i++] = rand() % 1000 / 500.0 - 1;
		pointsBuffer[i++] = rand() % 1000 / 500.0 - 1;
		pointsBuffer[i++] = 0.0f;
	}

	// Background shape
	pointsBuffer[i++] = -1.0f;
	pointsBuffer[i++] = -1.0f;
	pointsBuffer[i++] = 0.2f;
	
	pointsBuffer[i++] = 1.0f;
	pointsBuffer[i++] = -1.0f;
	pointsBuffer[i++] = 0.2f;
	
	pointsBuffer[i++] = 1.0f;
	pointsBuffer[i++] = 1.0f;
	pointsBuffer[i++] = 0.2f;
	
	pointsBuffer[i++] = -1.0f;
	pointsBuffer[i++] = 1.0f;
	pointsBuffer[i++] = 0.2f;
	
	// Bind point buffer to GL
	GLuint pointbuffer;
	glGenBuffers(1, &pointbuffer);
	glBindBuffer(GL_ARRAY_BUFFER, pointbuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(pointsBuffer), pointsBuffer, GL_STATIC_DRAW);

	do {
		// Clear the buffers for further use
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glUseProgram(progId);

		// background center
		glUniform2f(backgroundCenterId, 0.0, 0.0);
		glUniform1f(radId, radius);

		// point animation time
		glUniform1f(timeId, pointTime);
		pointTime += 0.0005;

		// Sets the size of the window
		glUniform2f(windowId, worldWidth, worldHeight);

		// point attribute buffer
		glEnableVertexAttribArray(ptId);
		glBindBuffer(GL_ARRAY_BUFFER, pointbuffer);
		glVertexAttribPointer(ptId, 3, GL_FLOAT, GL_FALSE, 0, (void*)0);

		// Make points
		glPointSize(34);
		glDrawArrays(GL_POINTS, 0, g_numP);

		// Make background
		glDrawArrays(GL_QUADS, g_numP, 4);

		glDisableVertexAttribArray(ptId);

		glfwSwapBuffers(window);
		glfwPollEvents();

	}
	while (!glfwWindowShouldClose(window));

	// Clean up buffers
	glDeleteBuffers(1, &pointbuffer);
	glDeleteProgram(progId);

	// Exit glfw
	glfwTerminate();

	return 0;
}
